import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import itertools

from sklearn import svm
from sklearn.neighbors import KNeighborsClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import train_test_split, cross_val_score
from sklearn.linear_model import LogisticRegression
from sklearn import metrics
import warnings
warnings.filterwarnings('ignore')


def plot_confusion_matrix(cm, classes=None, normalize=False, title='Confusion matrix', cmap=plt.cm.get_cmap('Blues')):
	"""
	This function prints and plots the confusion matrix.
	Normalization can be applied by setting `normalize=True`.
	"""
	if normalize:
		cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]
		print("Normalized confusion matrix")
	else:
		print('Confusion matrix, without normalization')
	
	print(cm)
	
	plt.figure()
	plt.imshow(cm, interpolation='nearest', cmap=cmap)
	plt.title(title)
	plt.colorbar()
	tick_marks = np.arange(len(classes))
	plt.xticks(tick_marks, classes, rotation=45)
	plt.yticks(tick_marks, classes)
	
	fmt = '.2f' if normalize else 'd'
	thresh = cm.max() / 2.
	for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
		plt.text(j, i, format(cm[i, j], fmt),
	          horizontalalignment="center",
	          color="white" if cm[i, j] > thresh else "black")
	
	plt.tight_layout()
	plt.ylabel('True label')
	plt.xlabel('Predicted label', labelpad=-5)
	plt.show()


def confidence_interval(metric, alpha=0.95):
	P_LOW_CONF=100 * (1.0 - alpha) / 2.0
	P_HIGH_CONF=100 * alpha + P_LOW_CONF
	
	lower, mean, upper = np.percentile(metric, [P_LOW_CONF, 50, P_HIGH_CONF])
	return lower, mean, upper


def data_analysis(data):
	columns = data.columns[:-1]
	nrows = len(columns) // 3 + 1
	for i, c in enumerate(columns, 1):
		plt.subplot(nrows, 3, i)
		plt.subplots_adjust(wspace=0.2, hspace=0.5)
		data[c].hist(bins=20, edgecolor='black')
		plt.title(c)
	plt.show()


def simplest_training(data):
	# TODO: Split the data into training (80%) the test (20%) sets. Stratify data.	
	X_train, X_test, y_train, y_test = None * 4
	
	classifiers = {
		'Linear SVM' : svm.SVC(kernel='linear'),
		#TODO: Fill in the different model instances
		'Radial SVM' : None,
		'Logistic Regression' : None,
		'KNN (3 classes)' : None,
		'Decision Tree' : None,
	}
	
	summary=[]
	for name, model in classifiers.items():
		model.fit(X_train, y_train)
		y_pred = model.predict(X_test)
		
		# TODO: Compute accuracy and confusion matrix
		acc = None
		cnf_matrix = None
		print('Model:', name)
		print(cnf_matrix)

		metric = []
		for _ in range(100):
			# TODO select with replacement a new test set the size of the original test set
			idx = None
			pred = model.predict(X_test.iloc[idx])
			test = y_test.iloc[idx]
			metric.append( metrics.accuracy_score(test, pred) )
		
		# TODO calculate the confidence interval
		lower, mean, upper = None
		print( 'CI', lower, mean, upper )

		# Plot non-normalized confusion matrix
# 		plot_confusion_matrix(cnf_matrix, classes=data.Class.unique(), title='Confusion matrix, without normalization')		
		summary.append(acc)

	models_dataframe = pd.DataFrame(summary, index=classifiers, columns=('Accuracy',))   
	print(models_dataframe)


def cross_validation_training(data):	
	classifiers = {
		'Linear SVM' : svm.SVC(kernel='linear'),
		#TODO: Fill in the different model instances
		'Radial SVM' : None,
		'Logistic Regression' : None,
		'KNN (3 classes)' : None,
		'Decision Tree' : None,
	}

	summary=[]
	for name, model in classifiers.items():
		# TODO: do a cross validation with 5 folds using accuracy as metric 
		scores = None
		print('Model:', name, scores)
		# TODO: calculate the CI based on quantiles
		lower, mean, upper = None
		summary.append([mean, lower, upper])

	models_dataframe = pd.DataFrame(summary, index=classifiers, columns=('Acc mean', 'Low', 'High'))
	print(models_dataframe.to_string())


def main():
	# Last column in data is the target called "Class"
	# TODO: Load CSV data into a pandas DataFrame. Choose either diabetes or iris data.
	data= None

	# Uncomment to visualize the data
# 	data_analysis(data)

# 	simplest_training(data)
	cross_validation_training(data)


if __name__ == '__main__':
	import time
	pd.set_option('display.width', 250)
	pd.set_option('display.max_columns', 100)
	np.set_printoptions(precision=2)
	pd.options.display.float_format = '{:.3f}'.format	
	s = time.time()
	main()
	e = time.time()
	print( 'Done in {:3f}'.format(e-s) )
